use egui::Key;
use async_trait::async_trait;
use tokio::task;

use std::sync::Arc;
use tokio::sync::Mutex;

use crate::app::metar::Metar;

use super::Tab;

#[derive(Debug)]
pub struct MetarTab {
    pub icaos: Vec<String>,
    pub metars: Vec<String>,
    pub metar: Arc<Mutex<Metar>>
}

impl Default for MetarTab {
    fn default() -> Self {
        Self {
            icaos: vec!["".to_string()],
            metars: vec![],
            metar: Arc::new(Mutex::new(Metar::default())),
        }
    }
}

impl PartialEq for MetarTab {
    fn eq(&self, other: &Self) -> bool {
        self.icaos == other.icaos && self.metars == other.metars
    }
}

impl Eq for MetarTab {}

#[async_trait]
impl Tab for MetarTab {
    async fn display_tab(&mut self, ctx: &egui::Context, frame: &egui::Frame) {
        egui::CentralPanel::default().frame(*frame).show(ctx, |ui|{
            // Given Key pressed, place focus on next item
            let new_icao_focus: bool = ui.input(|i| i.key_pressed(Key::Space));

            for icao in self.icaos.iter_mut() {
                ui.text_edit_singleline(icao);
            }

            if ui.button("Add More").clicked() || new_icao_focus {
                self.icaos.push("".to_string());

                // TODO: Improve error handling here
                // Spawn an async task to handle the async operation
                let metar_clone = Arc::clone(&self.metar);
                task::spawn(async move {
                    let metar = metar_clone.lock().await;
                    match metar.api.make_request(String::from("")).await {
                        Ok(response) => println!("{:?}", response),
                        Err(e) => eprintln!("Request failed: {:?}", e),
                    }
                });
            }
        });
    }
}
